﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Datos.AccesoDatos
{
    //Esta clase utiliza el patrón Singleton. De esta forma garantizamos una única instancia de la clase de acceso a datos.
    public class Festivos
    {
        private static Festivos _festivos;
        private GGPEntities _container;

        private Festivos()
        {
            _container = new GGPEntities();
        }

        public static Festivos Instanciar()
        {
            if (_festivos == null)
                _festivos = new Festivos();
            
            return _festivos;
        }

        public IEnumerable<Festivo> ObtenerFestivos()
        {
            return _container.Festivo.AsEnumerable();
        }

        public void CrearFestivo(DateTime fecha, string descripcion)
        {
            var festivo = new Festivo() { fecha = fecha, descripcion = descripcion };
            _container.Festivo.Add(festivo);
            _container.SaveChanges();
        }

        public void ModificarFestivo(DateTime fecha, string descripcion)
        {
            var festivo = ObtenerFestivo(fecha);

            if (festivo != null)
            {
                festivo.descripcion = descripcion;
                _container.SaveChanges();
            }
        }

        public void EliminarFestivo(DateTime fecha)
        {
            var festivo = ObtenerFestivo(fecha);

            if (festivo != null)
            {
                _container.Festivo.Remove(festivo);
                _container.SaveChanges();
            }
        }

        public Festivo ObtenerFestivo(DateTime fecha)
        {
            return _container.Festivo.Where(x => x.fecha == fecha).FirstOrDefault();        }
        }
}
