﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Datos.AccesoDatos
{
    //Esta clase utiliza el patrón Singleton. De esta forma garantizamos una única instancia de la clase de acceso a datos.
    public class HojasGastos
    {
        private static HojasGastos _hojasGastos;
        private GGPEntities _container;

        private HojasGastos()
        {
            _container = new GGPEntities();
        }

        public static HojasGastos Instanciar()
        {
            if (_hojasGastos == null)
                _hojasGastos = new HojasGastos();
            
            return _hojasGastos;
        }

        public IEnumerable<HojaGastos> ObtenerHojasGastos(int idUsuario)
        {
            return _container.HojaGastos.Where(x => x.idUsuario == idUsuario).AsEnumerable();
        }

        public void CrearHojaGastos(int idUsuario, int estado, Nullable<int> idResponsable, DateTime fechaCreacion,
            string comentarioResponsable, Nullable<DateTime> fechaCambioEstado)
        {
            var hojaGastos = new HojaGastos() { idUsuario = idUsuario, estado = estado, idResponsable = idResponsable, fechaCreacion = fechaCreacion, comentarioResponsable = comentarioResponsable, fechaCambioEstado= fechaCambioEstado };
            _container.HojaGastos.Add(hojaGastos);
            _container.SaveChanges();
        }


        public void ModificarHojaGastos(int idHojaGastos, int estado, Nullable<int> idResponsable,
            string comentarioResponsable, Nullable<DateTime> fechaCambioEstado)
        {
            var hojaGastos = ObtenerHojaGastos(idHojaGastos);

            if (hojaGastos != null)
            {
                hojaGastos.estado = estado;
                if (idResponsable != null)
                    hojaGastos.idResponsable = idResponsable;
                hojaGastos.comentarioResponsable = comentarioResponsable;
                if (fechaCambioEstado != null)
                    hojaGastos.fechaCambioEstado = fechaCambioEstado;
                _container.SaveChanges();
            }
        }

        public HojaGastos ObtenerHojaGastos(int idHojaGastos)
        {
            return _container.HojaGastos.Where(x => x.idHojaGastos == idHojaGastos).FirstOrDefault();       
        }

        public HojaGastos ObtenerUltimaHojaGastos(int idUsuario)
        {
            return _container.HojaGastos.Where(x => x.idUsuario == idUsuario).OrderByDescending(x => x.idHojaGastos).FirstOrDefault();
        }
    }
}
