﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Datos.AccesoDatos
{
    //Esta clase utiliza el patrón Singleton. De esta forma garantizamos una única instancia de la clase de acceso a datos.
    public class TiposGasto
    {
        private static TiposGasto _tiposGasto;
        private GGPEntities _container;

        private TiposGasto()
        {
            _container = new GGPEntities();
        }

        public static TiposGasto Instanciar()
        {
            if (_tiposGasto == null)
                _tiposGasto = new TiposGasto();
            
            return _tiposGasto;
        }

        public IEnumerable<TipoGasto> ObtenerTiposGasto()
        {
            return _container.TipoGasto.AsEnumerable();
        }

        public IEnumerable<TipoGasto> ObtenerTiposGastoActivos()
        {
            return _container.TipoGasto.Where(x => x.activo == true).AsEnumerable();
        }

        public bool ExisteTiposGasto(string nombre)
        {
            return _container.TipoGasto.Any(x => x.nombre == nombre);
        }

        public void CrearTipoGasto(string nombre, bool activo)
        {
            var tipoGasto = new TipoGasto() { nombre = nombre, activo = activo };
            _container.TipoGasto.Add(tipoGasto);
            _container.SaveChanges();
        }

        public void ModificarTipoGasto(int idTipoGasto, string nombre, bool activo)
        {
            var tipoGasto = ObtenerTipoGasto(idTipoGasto);

            if (tipoGasto != null)
            {
                tipoGasto.nombre = nombre;
                tipoGasto.activo = activo;
                _container.SaveChanges();
            }
        }

        public TipoGasto ObtenerTipoGasto(int idTipoGasto)
        {
            return _container.TipoGasto.Where(x => x.idTipoGasto == idTipoGasto).FirstOrDefault();
        }

        public TipoGasto ObtenerTipoGasto(string nombre)
        {
            return _container.TipoGasto.Where(x => x.nombre == nombre).FirstOrDefault();
        }
    }
}
