﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GGP.Administracion.Calendarios
{
    public partial class frmGestionCalendarios : Form
    {
        private Estados _estado;

        public frmGestionCalendarios()
        {
            InitializeComponent();

            var session = Negocio.FachadaNegocio.SesionUsuario.Instanciar();

             switch (session.Rol)
	        {
		        case Negocio.Modulos.TipoRolUsuario.Administrador:
                    _estado = Estados.NavegacionAdministrador;
                    break;
                case Negocio.Modulos.TipoRolUsuario.Empleado:
                     MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();  
                    break;
                case Negocio.Modulos.TipoRolUsuario.Responsable:
                     MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();  
                    break;
                default:
                    break;
	        }
            
            RealizarAccion(Acciones.CargarFormulario);   
        }

        private enum Estados
        {
            NavegacionAdministrador,
        }

        private enum Acciones
        {
            CargarFormulario,
            ConsultarFestivo,
            GuardarFestivo,
            Atras,
            Ayuda
        }
        
        private void RealizarAccion(Acciones accion)
        {
            try
            {
                switch (_estado)
                {
                    case Estados.NavegacionAdministrador:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.ConsultarFestivo:
                                RealizarConsultarFestivo();
                                break;
                            case Acciones.GuardarFestivo:
                                RealizarGuardarFestivo();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyuda();
                                break;
                            default:
                                break;
                        }
                        break;

                    default:
                        break;
                }

                HabilitarControles();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void HabilitarControles()
        {
            switch (_estado)
            {
                case Estados.NavegacionAdministrador:
                    btnGuardar.Visible = true;
                    btnGuardar.Visible = true;
                    break;

                default:
                    break;
            }
        }

        private void RealizarCargarFormulario()
        {
            dttCalendario.Value = DateTime.Now.Date;
        }

        private void RealizarConsultarFestivo()
        {
            var festivo = Negocio.FachadaNegocio.GestionFestivos.ObtenerFestivo(dttCalendario.Value);

            if (festivo != null)
            {
                chkActivo.Checked = true;
                txtDescripcion.Text = festivo.Descripcion;
            }
            else
            {
                chkActivo.Checked = false;
                txtDescripcion.Text = "";
            }
        }

        private void RealizarGuardarFestivo()
        {
            if (dttCalendario.Value != null)
	        {
                var festivo = new Negocio.Modulos.Festivo(dttCalendario.Value, txtDescripcion.Text);

                if (chkActivo.Checked)
                    Negocio.FachadaNegocio.GestionFestivos.CrearFestivo(festivo);
                else
                    Negocio.FachadaNegocio.GestionFestivos.EliminarFestivo(festivo);
                
                MessageBox.Show("Se guardado correctamente el día '" + festivo.Fecha.Date.ToShortDateString() + "'.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);

                RealizarAccion(Acciones.ConsultarFestivo);
	        }
            else
                MessageBox.Show("Debe seleccionar un día en el calendario.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void RealizarAtras()
        {
            this.Close();
        }

        private void RealizarAyuda()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Desde esta ventana puede consultar, crear, modificar o eliminar festivos.");
            ayuda.Append(" Debe seleccionar cualquier día en el calendario. Una vez seleccionado cargará la información referente a ese día.");
            ayuda.Append(" A continuación puede decir si el día es un festivo marcando el check de 'Activo' y añadir una descripción.");
            ayuda.Append(" Finalmente, puede seleccionar la opción 'Guardar' para guardar los cambios realizados sobre la fecha o 'Atrás' para salir del mantenmiento.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void dttCalendario_ValueChanged(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.ConsultarFestivo);
        }

        private void btnGuardar_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.GuardarFestivo);
        }

        private void btnAtrás_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Atras);
        }

        private void btnAyuda_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Ayuda);
        }

    }
}
