﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GGP.Administracion.Departamentos
{
    public partial class frmDetalleDepartamento : Form
    {
        private Estados _estado;
        private Nullable<int> _idDepartamento;

        public frmDetalleDepartamento(Nullable<int> idDepartamento)
        {
            InitializeComponent();

            var session = Negocio.FachadaNegocio.SesionUsuario.Instanciar();
            _idDepartamento = idDepartamento;

            switch (session.Rol)
            {
                case Negocio.Modulos.TipoRolUsuario.Administrador:
                    if (idDepartamento == null)
                        _estado = Estados.NuevoDepartamento;
                    else
                        _estado = Estados.DetalleDepartamento;
                    break;
                case Negocio.Modulos.TipoRolUsuario.Empleado:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                case Negocio.Modulos.TipoRolUsuario.Responsable:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                default:
                    break;
            }

            RealizarAccion(Acciones.CargarFormulario);
        }


        private enum Estados
        {
            NuevoDepartamento,
            DetalleDepartamento
        }

        private enum Acciones
        {
            CargarFormulario,
            GuardarDepartamento,
            Atras,
            Ayuda
        }

        private void RealizarAccion(Acciones accion)
        {
            try
            {
                switch (_estado)
                {
                    case Estados.NuevoDepartamento:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                break;
                            case Acciones.GuardarDepartamento:
                                RealizarGuardarDepartamento();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyudaNuevo();
                                break;
                            default:
                                break;
                        }
                        break;

                    case Estados.DetalleDepartamento:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.GuardarDepartamento:
                                RealizarGuardarDepartamento();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyudaDetalle();
                                break;
                            default:
                                break;
                        }
                        break;

                    default:
                        break;
                }

                HabilitarControles();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void HabilitarControles()
        {
            switch (_estado)
            {
                case Estados.NuevoDepartamento:
                    this.Text = "Nuevo departamento";
                    btnGuardar.Visible = true;
                    btnAtras.Visible = true;
                    break;

                case Estados.DetalleDepartamento:
                    this.Text = "Detalle departameto";
                    btnGuardar.Visible = true;
                    btnAtras.Visible = true;
                    break;

                default:
                    break;
            }
        }

        private void RealizarCargarFormulario()
        {
            if (_idDepartamento != null)
            {
                var departamento = Negocio.FachadaNegocio.GestionDepartamentos.ObtenerDepartamento(_idDepartamento.GetValueOrDefault());

                txtNombre.Text = departamento.Nombre;
                chkActivo.Checked = departamento.Activo;
                nudPorcentajePlusFestivo.Value = departamento.PorcentajePlusFestivo;
            }
        }

        private void RealizarGuardarDepartamento()
        {
            if (txtNombre.Text == "")
                MessageBox.Show("Debe informar un nombre para el departamento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            else
            {
                Negocio.Modulos.Departamento departamento;

                if (_idDepartamento == null)
                {
                    departamento = new Negocio.Modulos.Departamento(null, txtNombre.Text.Trim(), chkActivo.Checked, (int)nudPorcentajePlusFestivo.Value);
                    Negocio.FachadaNegocio.GestionDepartamentos.CrearDepartamento(departamento);
                    MessageBox.Show("Se creado correctamente el departamento '" + departamento.Nombre.Trim() + "'.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                }
                else
                {
                    if (Negocio.FachadaNegocio.GestionDepartamentos.DepartamentoRepetido(_idDepartamento.GetValueOrDefault(), txtNombre.Text))
                        MessageBox.Show("Ya existe un departmento con el nombre '" + txtNombre.Text.Trim() + "'.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    else
                    {
                        departamento = new Negocio.Modulos.Departamento(_idDepartamento.GetValueOrDefault(), txtNombre.Text.Trim(), chkActivo.Checked, (int)nudPorcentajePlusFestivo.Value);
                        Negocio.FachadaNegocio.GestionDepartamentos.ModificarDepartamento(departamento);
                        MessageBox.Show("Se ha modificado el departamento '" + departamento.Nombre.Trim() + "'.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        this.Close();
                    }
                }
            }
        }

        private void RealizarAtras()
        {
            this.Close();
        }

        private void RealizarAyudaNuevo()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Desde esta ventana puede crear un nuevo departamento.");
            ayuda.Append(" Debe indicar el nombre del departamento, si desea que esté activo y, finalmente, el porcentaje de incremento que desea que se haga sobre sus gastos cuando se produzcan en un día marcado como festivo.");
            ayuda.Append(" Finalmente, puede seleccionar la opción 'Guardar' para guardar el nuevo departamento o 'Atrás' para salir del mantenmiento.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void RealizarAyudaDetalle()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Desde esta ventana puede modificar un departamento ya existente.");
            ayuda.Append(" Puede modificar el nombre del departamento, si desea que esté activo y, finalmente, el porcentaje de incremento que desea que se haga sobre sus gastos cuando se produzcan en un día marcado como festivo.");
            ayuda.Append(" Finalmente, puede seleccionar la opción 'Guardar' para guardar la modificación del departamento o 'Atrás' para salir del mantenmiento.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void btnGuardar_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.GuardarDepartamento);
        }

        private void btnAtras_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Atras);
        }

        private void btnAyuda_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Ayuda);
        }

       

    }
}
