﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GGP.Administracion.TiposGastos
{
    public partial class DetalleTipoGasto : Form
    {
        private Estados _estado;
        private Nullable<int> _idTipoGasto;

        public DetalleTipoGasto(Nullable<int> idTipoGasto)
        {
            InitializeComponent();

            var session = Negocio.FachadaNegocio.SesionUsuario.Instanciar();
            _idTipoGasto = idTipoGasto;

            switch (session.Rol)
            {
                case Negocio.Modulos.TipoRolUsuario.Administrador:
                    if (idTipoGasto == null)
                        _estado = Estados.NuevoTipoGasto;
                    else
                        _estado = Estados.DetalleTipoGasto;
                    break;
                case Negocio.Modulos.TipoRolUsuario.Empleado:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                case Negocio.Modulos.TipoRolUsuario.Responsable:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                default:
                    break;
            }

            RealizarAccion(Acciones.CargarFormulario);
        }

        private enum Estados
        {
            NuevoTipoGasto,
            DetalleTipoGasto
        }

        private enum Acciones
        {
            CargarFormulario,
            GuardarTipoGasto,
            Atras,
            Ayuda
        }

        private void RealizarAccion(Acciones accion)
        {
            try
            {
                switch (_estado)
                {
                    case Estados.NuevoTipoGasto:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.GuardarTipoGasto:
                                RealizarGuardarTipoGasto();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyudaNuevo();
                                break;
                            default:
                                break;
                        }
                        break;

                    case Estados.DetalleTipoGasto:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.GuardarTipoGasto:
                                RealizarGuardarTipoGasto();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyudaDetalle();
                                break;
                            default:
                                break;
                        }
                        break;

                    default:
                        break;
                }

                HabilitarControles();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void HabilitarControles()
        {
            switch (_estado)
            {
                case Estados.NuevoTipoGasto:
                    this.Text = "Nuevo tipo de gasto";
                    btnGuardar.Visible = true;
                    btnAtras.Visible = true;
                    break;

                case Estados.DetalleTipoGasto:
                    this.Text = "Detalle tipo de gasto";
                    btnGuardar.Visible = true;
                    btnAtras.Visible = true;
                    break;

                default:
                    break;
            }
        }

        private void RealizarCargarFormulario()
        {
            if (_idTipoGasto != null)
            {
                var tipoGasto = Negocio.FachadaNegocio.GesitonTiposGasto.ObtenerTipoGasto(_idTipoGasto.GetValueOrDefault());

                txtNombre.Text = tipoGasto.Nombre;
                chkActivo.Checked = tipoGasto.Activo;
            }
        }

        private void RealizarGuardarTipoGasto()
        {
            if (txtNombre.Text == "")
                MessageBox.Show("Debe informar un nombre para el tipo de gasto.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            else
            {
                Negocio.Modulos.TipoGasto tipoGasto;

                if (_idTipoGasto == null)
                {
                    tipoGasto = new Negocio.Modulos.TipoGasto(null, txtNombre.Text.Trim(), chkActivo.Checked);
                    Negocio.FachadaNegocio.GesitonTiposGasto.CrearTipoGasto(tipoGasto);
                    MessageBox.Show("Se creado correctamente el tipo de gasto '" + tipoGasto.Nombre.Trim() + "'.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                }
                else
                {
                    if (Negocio.FachadaNegocio.GesitonTiposGasto.TipoGastoRepetido(_idTipoGasto.GetValueOrDefault(), txtNombre.Text))
                        MessageBox.Show("Ya existe un tipo de gasto con el nombre '" + txtNombre.Text.Trim() + "'.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    else
                    {
                        tipoGasto = new Negocio.Modulos.TipoGasto(_idTipoGasto.GetValueOrDefault(), txtNombre.Text.Trim(), chkActivo.Checked);
                        Negocio.FachadaNegocio.GesitonTiposGasto.ModificarTipoGasto(tipoGasto);
                        MessageBox.Show("Se modificado correctamente el tipo de gasto '" + tipoGasto.Nombre.Trim() + "'.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        this.Close();
                    }
                }
            }
        }

        private void RealizarAtras()
        {
            this.Close();
        }

        private void RealizarAyudaNuevo()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Desde esta ventana puede crear un nuevo tipo de gasto.");
            ayuda.Append(" Debe indicar el nombre del tipo de gasto y si desea que esté activo.");
            ayuda.Append(" Finalmente, puede seleccionar la opción 'Guardar' para guardar el nuevo tipo de gasto o 'Atrás' para salir del mantenmiento.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void RealizarAyudaDetalle()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Desde esta ventana puede modificar un tipo de gasto ya existente.");
            ayuda.Append(" Puede modificar el nombre del tipo de gasto e indicar si desea que esté activo.");
            ayuda.Append(" Finalmente, puede seleccionar la opción 'Guardar' para guardar la modificación del tipo de gasto o 'Atrás' para salir del mantenmiento.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void btnGuardar_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.GuardarTipoGasto);
        }

        private void btnAtras_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Atras);
        }

        private void btnAyuda_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Ayuda);
        }

       



    }
}
