﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GGP.Administracion.TiposGastos
{
    public partial class ListaTiposGastos : Form
    {
        private Estados _estado;

        public ListaTiposGastos()
        {
            InitializeComponent();

            var session = Negocio.FachadaNegocio.SesionUsuario.Instanciar();

            switch (session.Rol)
            {
                case Negocio.Modulos.TipoRolUsuario.Administrador:
                    _estado = Estados.NavegacionAdministrador;
                    break;
                case Negocio.Modulos.TipoRolUsuario.Empleado:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                case Negocio.Modulos.TipoRolUsuario.Responsable:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                default:
                    break;
            }

            RealizarAccion(Acciones.CargarFormulario);
        }

        private enum Estados
        {
            NavegacionAdministrador
        }

        private enum Acciones
        {
            CargarFormulario,
            MostrarDetalleTipoGasto,
            Atras,
            Ayuda
        }

        private void RealizarAccion(Acciones accion)
        {
            try
            {
                switch (_estado)
                {
                    case Estados.NavegacionAdministrador:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.MostrarDetalleTipoGasto:
                                RealizarMostrarDetalleTipoGasto();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyuda();
                                break;
                            default:
                                break;
                        }
                        break;

                    default:
                        break;
                }

                HabilitarControles();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void HabilitarControles()
        {
            switch (_estado)
            {
                case Estados.NavegacionAdministrador:
                    btnDetalle.Visible = true;
                    btnAtras.Visible = true;
                    break;

                default:
                    break;
            }
        }

        private void RealizarCargarFormulario()
        {
            ICollection<Negocio.Modulos.TipoGasto> tiposGasto;
            tiposGasto = Negocio.FachadaNegocio.GesitonTiposGasto.ObtenerTiposGastos();
            tipoGastoBindingSource.DataSource = tiposGasto;
        }

        private void RealizarMostrarDetalleTipoGasto()
        {
            if (grdTiposGastos.SelectedRows.Count == 1)
            {
                var frmDetalleTipoGasto = new DetalleTipoGasto((int)grdTiposGastos.SelectedRows[0].Cells["idTipoGastoDataGridViewTextBoxColumn"].Value);
                frmDetalleTipoGasto.ShowDialog();
                RealizarAccion(Acciones.CargarFormulario);
            }
            else
                MessageBox.Show("Debe seleccionar un tipo de gasto.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void RealizarAtras()
        {
            this.Close();
        }

        private void RealizarAyuda()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Desde esta ventana puede consultar la lista de tipos de gastos creados.");
            ayuda.Append(" Si selecciona un tipo de gasto puede ver el detalle seleccionando la opción 'Detalle'. Una vez dentro del detalle puede modificar los datos del tipo de gasto.");
            ayuda.Append(" Si desea volver atrás puede seleccionar la opción 'Atrás'.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void btnDetalle_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.MostrarDetalleTipoGasto);
        }

        private void btnAtras_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Atras);
        }

        private void btnAyuda_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Ayuda);
        }
    }
}
