﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GGP.Administracion.UsuariosRoles
{
    public partial class DetalleUsuario : Form
    {
        private Estados _estado;
        private Nullable<int> _idUsuario;

        public DetalleUsuario(Nullable<int> idUsuario)
        {
            InitializeComponent();

            var session = Negocio.FachadaNegocio.SesionUsuario.Instanciar();
            _idUsuario = idUsuario;

            switch (session.Rol)
            {
                case Negocio.Modulos.TipoRolUsuario.Administrador:
                    if (idUsuario == null)
                        _estado = Estados.NuevoUsuario;
                    else
                        _estado = Estados.DetalleUsuario;
                    break;
                case Negocio.Modulos.TipoRolUsuario.Empleado:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                case Negocio.Modulos.TipoRolUsuario.Responsable:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                default:
                    break;
            }

            RealizarAccion(Acciones.CargarFormulario);
        }

        private enum Estados
        {
            NuevoUsuario,
            DetalleUsuario
        }

        private enum Acciones
        {
            CargarFormulario,
            GuardarUsuario,
            Atras,
            Ayuda
        }

        private void RealizarAccion(Acciones accion)
        {
            try
            {
                switch (_estado)
                {
                    case Estados.NuevoUsuario:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.GuardarUsuario:
                                RealizarGuardarUsuario();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyudaNuevo();
                                break;
                            default:
                                break;
                        }
                        break;

                    case Estados.DetalleUsuario:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.GuardarUsuario:
                                RealizarGuardarUsuario();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyudaDetalle();
                                break;
                            default:
                                break;
                        }
                        break;

                    default:
                        break;
                }

                HabilitarControles();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void HabilitarControles()
        {
            switch (_estado)
            {
                case Estados.NuevoUsuario:
                    this.Text = "Nuevo usuario";
                    btnGuardar.Visible = true;
                    btnAtras.Visible = true;
                    break;

                case Estados.DetalleUsuario:
                    this.Text = "Detalle usuario";
                    btnGuardar.Visible = true;
                    btnAtras.Visible = true;
                    txtContrasena.PasswordChar = '*';
                    break;
                
                default:
                    break;
            }
        }

        private void RealizarCargarFormulario()
        {
            var departamentos = Negocio.FachadaNegocio.GestionDepartamentos.ObtenerDepartamentos();
            departamentoBindingSource.DataSource = departamentos;

            var rolesUsuario = Negocio.FachadaNegocio.GestionRolesUsuario.ObtenerRolesUsuario();
            rolUsuarioBindingSource.DataSource = rolesUsuario;

            if (_idUsuario != null)
            {
                var usuario = Negocio.FachadaNegocio.GestionUsuarios.ObtenerUsuario(_idUsuario.GetValueOrDefault());

                txtNombre.Text = usuario.Nombre;
                txtApellidos.Text = usuario.Apellidos;
                txtNombreLogin.Text = usuario.NombreLogin;
                txtContrasena.Text = usuario.Contrasena;

                cmbDepartamento.SelectedValue = usuario.IdDepartamento;
                cmbRol.SelectedValue = (int)usuario.Rol;

                chkActivo.Checked = usuario.Activo;
            }
        }

        private void RealizarGuardarUsuario()
        {
            String texto = "";

            if (txtNombre.Text == "")
                texto = "Nombre";

            if (txtApellidos.Text == "")
            {
                if (texto != "")
                    texto += ", ";
                
                texto += ", Apellidos";
            }

            if (txtNombreLogin.Text == "")
            {
                if (texto != "")
                    texto += ", ";

                texto += ", NombreLogin";
            }

            if (txtContrasena.Text == "")
            {
                if (texto != "")
                    texto += ", ";

                texto += ", Contraseña";
            }

            if (cmbDepartamento.SelectedValue == null)
            {
                if (texto != "")
                    texto += ", ";

                texto += ", Departamento";
            }

            if (cmbRol.SelectedValue == null)
            {
                if (texto != "")
                    texto += ", ";

                texto += ", Rol";
            }

            if (texto == "")
            {
                Negocio.Modulos.Usuario usuario;

                if (_idUsuario == null)
                {
                    usuario = new Negocio.Modulos.Usuario(null, txtNombre.Text.Trim(), txtApellidos.Text.Trim(),
                        txtNombreLogin.Text.Trim(), txtContrasena.Text.Trim(), chkActivo.Checked, (int)cmbDepartamento.SelectedValue, (Negocio.Modulos.TipoRolUsuario)cmbRol.SelectedValue);
                    Negocio.FachadaNegocio.GestionUsuarios.CrearUsuario(usuario);
                    MessageBox.Show("Se ha creado el usuario '" + usuario.Nombre.Trim() + "'.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                }
                else
                {
                    if (Negocio.FachadaNegocio.GestionUsuarios.LoginRepetido(_idUsuario.GetValueOrDefault(), txtNombreLogin.Text.Trim()))
                        MessageBox.Show("Ya existe un usuario con el nombre '" + txtNombreLogin.Text.Trim() + "'.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    else
                    {
                        usuario = new Negocio.Modulos.Usuario(_idUsuario.GetValueOrDefault(), txtNombre.Text.Trim(), txtApellidos.Text.Trim(),
                            txtNombreLogin.Text.Trim(), txtContrasena.Text.Trim(), chkActivo.Checked, (int)cmbDepartamento.SelectedValue, (Negocio.Modulos.TipoRolUsuario)cmbRol.SelectedValue);
                        Negocio.FachadaNegocio.GestionUsuarios.ModificarUsuario(usuario);
                        MessageBox.Show("Se ha modificado el usuario '" + usuario.Nombre.Trim() + "'.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        this.Close();
                    }
                }
            }
            else
                MessageBox.Show("Debe informar los campos: " + texto + ".", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            
        }

        private void RealizarAtras()
        {
            this.Close();
        }

        private void RealizarAyudaNuevo()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Desde esta ventana puede crear un nuevo usuario.");
            ayuda.Append(" Debe indicar todos los campos que se solicitan.");
            ayuda.Append(" Finalmente, puede seleccionar la opción 'Guardar' para guardar el nuevo usuario o 'Atrás' para salir del mantenmiento.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void RealizarAyudaDetalle()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Desde esta ventana puede modificar un usuario ya existente.");
            ayuda.Append(" Puede modificar cualquier de los campos que se muestran.");
            ayuda.Append(" Finalmente, puede seleccionar la opción 'Guardar' para guardar la modificación del usuario o 'Atrás' para salir del mantenmiento.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void btnGuardar_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.GuardarUsuario);
        }

        private void btnAtras_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Atras);
        }

        private void btnAyuda_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Ayuda);
        }

       
    }
}
