﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GGP.Gestion
{
    public partial class frmAprobarRechazarHoja : Form
    {
        private int _idHojaGastos;

        public frmAprobarRechazarHoja(int idHojaGastos)
        {
            InitializeComponent();

            var sesion = Negocio.FachadaNegocio.SesionUsuario.Instanciar();

            switch (sesion.Rol)
            {
                case Negocio.Modulos.TipoRolUsuario.Administrador:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                case Negocio.Modulos.TipoRolUsuario.Empleado:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                case Negocio.Modulos.TipoRolUsuario.Responsable:
                    break;
                default:
                    break;
            }

            _idHojaGastos = idHojaGastos;
        }

        private void btnGuardar_Click(object sender, EventArgs e)
        {
            try
            {
                if (radAprobar.Checked || radRechazar.Checked)
                {
                    if (radRechazar.Checked && txtObservaciones.Text == "")
                        MessageBox.Show("Para rechazar la hoja de gastos debe indicar un motivo en el campo observaciones.", "Información", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    else
                    {
                        var hojaGastos = Negocio.FachadaNegocio.GestionHojasGastos.ObtenerHojaGastos(_idHojaGastos);
                        var sesion = Negocio.FachadaNegocio.SesionUsuario.Instanciar();

                        if (radAprobar.Checked)
                            hojaGastos.Estado = Negocio.Modulos.EstadoHojaGastos.Aceptada;
                        else
                            hojaGastos.Estado = Negocio.Modulos.EstadoHojaGastos.Rechazada;

                        hojaGastos.IdResponsable = sesion.IdUsuario;
                        hojaGastos.ComentarioResponsable = txtObservaciones.Text;
                        hojaGastos.FechaCambioEstado = DateTime.Now;

                        Negocio.FachadaNegocio.GestionHojasGastos.ModificarHojaGastos(hojaGastos);
                        MessageBox.Show("La hoja de gastos ha quedado en estado " + hojaGastos.Estado.ToString() + ".", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        this.Close();
                    }
                }
                else
                    MessageBox.Show("Para guardar primero debe indicar si aprueba/cancela la hoja de gastos.", "Información", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void btnAyuda_Click(object sender, EventArgs e)
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Seleccione si desea aprobar o rechazar la hoja de gastos, introduzca un comentarios,"  );
            ayuda.Append(" y seleccione la opción 'Guardar' para registrar el cambio en el sistema.");
            ayuda.Append(" Si desea salir sin guardar puede seleccionar la opción 'Atrás'");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void btnAtras_Click(object sender, EventArgs e)
        {
            this.Close();
        }




    }
}
