﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GGP.Gestion
{
    public partial class fmrDetalleGasto : Form
    {
        private Estados _estado;
        private Negocio.Modulos.DetalleHojaGastos _detalleHojaGastos;
        private byte[] _archivoAdjunto;

        public Negocio.Modulos.DetalleHojaGastos DetalleHoja { get; set; }

        public fmrDetalleGasto(Negocio.Modulos.DetalleHojaGastos detalleHojaGastos)
        {
            InitializeComponent();

            var sesion = Negocio.FachadaNegocio.SesionUsuario.Instanciar();
            _detalleHojaGastos = detalleHojaGastos;

            if (sesion.Rol == Negocio.Modulos.TipoRolUsuario.Administrador)
            {
                MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                this.Close();
            }
            else
            {
                if (detalleHojaGastos == null)
                    _estado = Estados.NuevoDetalle;
                else
                    _estado = Estados.VisualizacionDetalle;
            }

            RealizarAccion(Acciones.CargarFormulario);
        }

        private enum Estados
        {
            NuevoDetalle,
            VisualizacionDetalle
        }

        private enum Acciones
        {
            CargarFormulario,
            AdjuntarArchivo,
            AbrirArchivoAdjunto,
            GuardarDetalle,
            Atras,
            Ayuda
        }

        private void RealizarAccion(Acciones accion)
        {
            try
            {
                switch (_estado)
                {
                    case Estados.NuevoDetalle:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.AdjuntarArchivo:
                                RealizarAdjuntarArchivo();
                                break;
                            case Acciones.GuardarDetalle:
                                RealizarGuardarDetalle();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyudaNueva();
                                break;
                            default:
                                break;
                        }
                        break;

                    case Estados.VisualizacionDetalle:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.AbrirArchivoAdjunto:
                                RealizarAbrirArchivoAdjuto();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyudaDetalle();
                                break;
                            default:
                                break;
                        }
                        break;

                    default:
                        break;

                }

                HabilitarControles();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void HabilitarControles()
        {
            switch (_estado)
            {
                case Estados.NuevoDetalle:
                    this.Text = "Nuevo detalle de gasto";
                    btnAdjuntar.Visible = true;
                    btnVerAdjunto.Visible = false;
                    btnGuardar.Visible = true;
                    btnAtras.Visible = true;

                    txtDescripcion.ReadOnly = false;
                    txtObservaciones.ReadOnly = false;
                    txtImporte.ReadOnly = false;
                    txtImporteCalculado.ReadOnly = false;
                    dttFechaInicio.Enabled = true;
                    dttFechaFin.Enabled = true;
                    break;

                case Estados.VisualizacionDetalle:
                    this.Text = "Detalle de gasto";
                    btnAdjuntar.Visible = false;
                    btnVerAdjunto.Visible = true;
                    btnGuardar.Visible = false;
                    btnAtras.Visible = true;

                    cmbTipoGasto.Enabled = false;
                    txtDescripcion.ReadOnly = true;
                    txtObservaciones.ReadOnly = true;
                    txtImporte.ReadOnly = true;
                    txtImporteCalculado.ReadOnly = true;
                    dttFechaInicio.Enabled = false;
                    dttFechaFin.Enabled = false;
                    break;

                default:
                    break;
            }
        }

        private void RealizarCargarFormulario()
        {
            
            if (_detalleHojaGastos != null)
            {
                var tiposGasto = Negocio.FachadaNegocio.GesitonTiposGasto.ObtenerTiposGastos();
                tipoGastoBindingSource.DataSource = tiposGasto;

                txtDescripcion.Text = _detalleHojaGastos.Descripcion;
                txtObservaciones.Text = _detalleHojaGastos.Observaciones;
                dttFechaInicio.Value = _detalleHojaGastos.FechaInicio;
                dttFechaFin.Value = _detalleHojaGastos.FechaFin;
                cmbTipoGasto.SelectedValue = _detalleHojaGastos.IdTipoGasto;
                txtImporte.Text = _detalleHojaGastos.Importe.ToString();
                txtImporteCalculado.Text = _detalleHojaGastos.ImporteCalculado.ToString();
                _archivoAdjunto = _detalleHojaGastos.ImagenAdjunta;

                if (_archivoAdjunto == null)
                    btnVerAdjunto.Enabled = false;
            }
            else
            {
                var tiposGastoActivos = Negocio.FachadaNegocio.GesitonTiposGasto.ObtenerTiposGastosActivos();
                tipoGastoBindingSource.DataSource = tiposGastoActivos;

                dttFechaInicio.Value = DateTime.Now.Date;
                dttFechaFin.Value = DateTime.Now.Date;
            }
        }

        private void RealizarAdjuntarArchivo()
        {
            ofdAdjuntarArchivo.Filter = "Image files (*.jpg, *.jpeg, *.jpe, *.jfif, *.png) | *.jpg; *.jpeg; *.jpe; *.jfif; *.png";
            var dialogResult = ofdAdjuntarArchivo.ShowDialog();

            if (dialogResult == System.Windows.Forms.DialogResult.OK && ofdAdjuntarArchivo.CheckFileExists)
            {
                var binaryReader = new BinaryReader(new FileStream(ofdAdjuntarArchivo.FileName, FileMode.Open));
                _archivoAdjunto = binaryReader.ReadBytes((int)binaryReader.BaseStream.Length);
                binaryReader.Close();
            }
        }

        private void RealizarAbrirArchivoAdjuto()
        {
            SaveFileDialog saveDialog = new SaveFileDialog();
            saveDialog.Filter = "JPG (*.jpg) | *.jpg";
            var dialogResult = saveDialog.ShowDialog();

            if (dialogResult == System.Windows.Forms.DialogResult.OK)
            {
                var archivoAdjunto = ByteArrayToImage(_archivoAdjunto);
                archivoAdjunto.Save(saveDialog.FileName.ToString(), System.Drawing.Imaging.ImageFormat.Jpeg);
                System.Diagnostics.Process.Start(saveDialog.FileName);
            }

        }

        private void RealizarGuardarDetalle()
        {
            if (dttFechaInicio.Value > dttFechaFin.Value)
            {
                MessageBox.Show("La fecha de fin no puede ser inferior a la fecha de inicio.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                String camposNoInformados = comprobarCampos();
                if (camposNoInformados == "")
                {
                    var sesion = Negocio.FachadaNegocio.SesionUsuario.Instanciar();

                    double importe;
                    double.TryParse(txtImporte.Text, out importe);

                    if (importe > 0)
                    {
                        var importeCalculado = Negocio.FachadaNegocio.GestionDetallesHojaGastos.CalcularImporte(sesion.IdDepartamento, importe, dttFechaInicio.Value, dttFechaFin.Value);
                        txtImporteCalculado.Text = importeCalculado.ToString();

                        DetalleHoja = new Negocio.Modulos.DetalleHojaGastos(null, 0, (int)cmbTipoGasto.SelectedValue, txtDescripcion.Text, txtObservaciones.Text,
                            dttFechaInicio.Value, dttFechaFin.Value, importe, importeCalculado, _archivoAdjunto);

                        this.Close();
                    }
                    else
                        MessageBox.Show("Debe indicar un importe numérico superior a 0.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Debe indicar un valor para los siguientes campos: " + camposNoInformados + ".", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }

            }
        }

        private void RealizarAtras()
        {
            this.Close();
        }

        private void RealizarAyudaNueva()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Desde esta ventana puede crear un nuevo detalle de gasto. Un detalle de gasto es un solo concepto (por ejemplo: vuelo, comida, taxi...).");
            ayuda.Append(" Debe rellenar todos los campos que se le solicitan a excepción del 'Importe calculado' Este campo se autocalculará teniendo el cuenta si el gasto se ha producido en un día festivo o no.");
            ayuda.Append(" Para guardar el detalle de gasto seleccione el botón 'Gurdar'. Para salir sin guardar, seleccione el botón 'Atrás'.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void RealizarAyudaDetalle()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Desde esta ventana puede revisar un detalle de gasto. Un detalle de gasto es un solo concepto (por ejemplo: vuelo, comida, taxi...).");
            ayuda.Append(" Para salir seleccione el botón 'Atrás'.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private String comprobarCampos()
        {
            String texto = "";

            if (cmbTipoGasto.SelectedValue == null)
                texto = "Tipo de gasto";

            if (txtDescripcion.Text == "")
            {
                if (texto != "")
                    texto += " ,";
                texto += "Descripción";
            }

            if (txtObservaciones.Text == "")
            {
                if (texto != "")
                    texto += " ,";
                texto += "Observaciones";
            }

            if (txtImporte.Text == "")
            {
                if (texto != "")
                    texto += " ,";
                texto += "Importe";
            }

            return texto;
        }

        public byte[] ImageToByteArray(Image imageIn)
        {
            MemoryStream ms = new MemoryStream();
            imageIn.Save(ms, System.Drawing.Imaging.ImageFormat.Gif);
            return ms.ToArray();
        }

        public Image ByteArrayToImage(byte[] byteArrayIn)
        {
            MemoryStream ms = new MemoryStream(byteArrayIn);
            Image returnImage = Image.FromStream(ms);
            return returnImage;
        }

        private void btnAdjuntar_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.AdjuntarArchivo);
        }

        private void btnGuardar_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.GuardarDetalle);
        }

        private void btnAtras_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Atras);
        }

        private void btnAyuda_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Ayuda);
        }

        private void btnVerImagen_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.AbrirArchivoAdjunto);
        }

    }
}
