﻿using Negocio.Modulos;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GGP.Gestion
{
    public partial class DetalleHojaGastos : Form
    {
        private Estados _estado;
        private Nullable<int> _idHojaGastos;

        public DetalleHojaGastos(Nullable<int> idHojaGastos)
        {
            InitializeComponent();
            var sesion = Negocio.FachadaNegocio.SesionUsuario.Instanciar();

            switch (sesion.Rol)
            {
                case Negocio.Modulos.TipoRolUsuario.Administrador:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                case Negocio.Modulos.TipoRolUsuario.Empleado:
                    if (idHojaGastos.HasValue)
                        _estado = Estados.NavegacionEmpleado;
                    else
                        _estado = Estados.Creacion;
                    break;
                case Negocio.Modulos.TipoRolUsuario.Responsable:
                    if (idHojaGastos.HasValue)
                        _estado = Estados.NavegacionResponsable;
                    else
                        _estado = Estados.Creacion;
                    break;
                default:
                    break;
            }

            _idHojaGastos = idHojaGastos;
            RealizarAccion(Acciones.CargarFormulario);
        }

        private enum Estados
        {
            Creacion,
            NavegacionEmpleado,
            NavegacionResponsable,
            NavegacionResponsableHojaGestionada
        }

        private enum Acciones
        {
            CargarFormulario,
            NuevoDetalle,
            EliminarDetalle,
            MostrarDetalle,
            GuardarHoja,
            CambiarEstadoHoja,
            Atras,
            Ayuda
        }

        private void RealizarAccion(Acciones accion)
        {
            try
            {
                switch (_estado)
                {
                    case Estados.Creacion:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.NuevoDetalle:
                                RealizarNuevoDetalle();
                                break;
                            case Acciones.EliminarDetalle:
                                RealizarEliminarDetalle();
                                break;
                            case Acciones.MostrarDetalle:
                                RealizarMostrarDetalle();
                                break;
                            case Acciones.GuardarHoja:
                                RealizarGuardarHoja();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyuda();
                                break;
                            default:
                                break;
                        }
                        break;

                    case Estados.NavegacionEmpleado:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.MostrarDetalle:
                                RealizarMostrarDetalle();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyuda();
                                break;
                            default:
                                break;
                        }
                        break;

                    case Estados.NavegacionResponsable:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.MostrarDetalle:
                                RealizarMostrarDetalle();
                                break;
                            case Acciones.CambiarEstadoHoja:
                                RealizarCambiarEstadoHoja();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyuda();
                                break;
                            default:
                                break;
                        }
                        break;

                    case Estados.NavegacionResponsableHojaGestionada:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;
                            case Acciones.MostrarDetalle:
                                RealizarMostrarDetalle();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyuda();
                                break;
                            default:
                                break;
                        }
                        break;

                    default:
                        break;
                }

                HabilitarControles();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void HabilitarControles()
        {
            switch (_estado)
            {
                case Estados.Creacion:
                    btnNuevoDetalle.Visible = true;
                    btnEliminarDetalle.Visible = true;
                    btnGuardar.Visible = true;
                    btnAceptarRechazar.Visible = false;
                    break;

                case Estados.NavegacionEmpleado:
                    btnNuevoDetalle.Visible = false;
                    btnEliminarDetalle.Visible = false;
                    btnGuardar.Visible = false;
                    btnAceptarRechazar.Visible = false;
                    break;

                case Estados.NavegacionResponsable:
                    btnNuevoDetalle.Visible = false;
                    btnEliminarDetalle.Visible = false;
                    btnGuardar.Visible = false;
                    btnAceptarRechazar.Visible = true;
                    break;

                case Estados.NavegacionResponsableHojaGestionada:
                    btnNuevoDetalle.Visible = false;
                    btnEliminarDetalle.Visible = false;
                    btnGuardar.Visible = false;
                    btnAceptarRechazar.Visible = false;
                    break;

                default:
                    break;
            }

        }

        private void RealizarCargarFormulario()
        {
            Usuario empleado;

            var hojaGastos = Negocio.FachadaNegocio.GestionHojasGastos.ObtenerHojaGastos(_idHojaGastos.GetValueOrDefault());

            if (hojaGastos != null)
            {
                empleado = Negocio.FachadaNegocio.GestionUsuarios.ObtenerUsuario(hojaGastos.IdUsuario);
                var responsable = Negocio.FachadaNegocio.GestionUsuarios.ObtenerUsuario(hojaGastos.IdResponsable.GetValueOrDefault());
                var detallesHojaGastos = Negocio.FachadaNegocio.GestionDetallesHojaGastos.ObtenerDetallesHojaGastos(_idHojaGastos.GetValueOrDefault());

                txtEmpleado.Text = empleado.Nombre + " " + empleado.Apellidos;

                if (responsable != null)
                    txtResponsable.Text = responsable.Nombre + " " + responsable.Apellidos;

                txtEstado.Text = hojaGastos.Estado.ToString();

                detalleHojaGastosBindingSource.DataSource = detallesHojaGastos;

                if (_estado == Estados.NavegacionResponsable & hojaGastos.Estado != Negocio.Modulos.EstadoHojaGastos.Pendiente)
                    _estado = Estados.NavegacionResponsableHojaGestionada;
            }
            else
            {
                detalleHojaGastosBindingSource.DataSource = new Collection<Negocio.Modulos.DetalleHojaGastos>();
                var session = Negocio.FachadaNegocio.SesionUsuario.Instanciar();
                empleado = Negocio.FachadaNegocio.GestionUsuarios.ObtenerUsuario(session.IdUsuario);
                txtEmpleado.Text = empleado.NombreApellidos;
            }
        }

        private void RealizarNuevoDetalle()
        {
            var frmDetalleGasto = new fmrDetalleGasto(null);
            frmDetalleGasto.ShowDialog();

            if (frmDetalleGasto.DetalleHoja != null)
            {
                var detallesHojaGastos = (Collection<Negocio.Modulos.DetalleHojaGastos>)detalleHojaGastosBindingSource.DataSource;
                detallesHojaGastos.Add(frmDetalleGasto.DetalleHoja);
                detalleHojaGastosBindingSource.ResetBindings(false);    
            }
        }

        private void RealizarEliminarDetalle()
        {
            if (grdDetalles.SelectedRows.Count == 1)
                grdDetalles.Rows.Remove(grdDetalles.SelectedRows[0]);
            else
                MessageBox.Show("Debe seleccionar un detalle para poder eliminarlo.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void RealizarMostrarDetalle()
        {
            if (grdDetalles.SelectedRows.Count == 1)
            {

                var frmDetalleGasto = new fmrDetalleGasto((Negocio.Modulos.DetalleHojaGastos)grdDetalles.SelectedRows[0].DataBoundItem);
                frmDetalleGasto.ShowDialog();
            }
            else
                MessageBox.Show("Debe seleccionar un detalle para poder eliminarlo.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void RealizarGuardarHoja()
        {
            if (grdDetalles.Rows.Count == 0)
                MessageBox.Show("Debe introducir algún detalle para poder guardar una hoja de gastos.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            else
            {
                var sesion = Negocio.FachadaNegocio.SesionUsuario.Instanciar();
                var hojaGastos = new Negocio.Modulos.HojaGastos(null, sesion.IdUsuario, Negocio.Modulos.EstadoHojaGastos.Pendiente, null,
                    DateTime.Now, "", null);

                Negocio.FachadaNegocio.GestionHojasGastos.CrearHojaGastos(hojaGastos);

                hojaGastos = Negocio.FachadaNegocio.GestionHojasGastos.ObtenerUltimaHojaGastos(sesion.IdUsuario);
                foreach (var row in (Collection<Negocio.Modulos.DetalleHojaGastos>)detalleHojaGastosBindingSource.DataSource)
                {
                    var detalleHojaGastos = new Negocio.Modulos.DetalleHojaGastos(null, hojaGastos.IdHojaGastos.GetValueOrDefault(), row.IdTipoGasto, row.Descripcion,
                        row.Observaciones, row.FechaInicio, row.FechaFin, row.Importe, row.ImporteCalculado, row.ImagenAdjunta);

                    Negocio.FachadaNegocio.GestionDetallesHojaGastos.CrearDetalleHojaGastos(detalleHojaGastos);
                }

                MessageBox.Show("Se ha creado correctamente la hoja de gastos.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                this.Close();
            }
        }

        private void RealizarCambiarEstadoHoja()
        {
            if (_idHojaGastos != null)
            {
                var frmAprobarRechazarHoja = new frmAprobarRechazarHoja(_idHojaGastos.GetValueOrDefault());
                frmAprobarRechazarHoja.ShowDialog();

                RealizarCargarFormulario();
            }
        }

        private void RealizarAtras()
        {
            this.Close();
        }

        private void RealizarAyuda()
        {
            switch (_estado)
            {
                case Estados.Creacion:
                    var ayuda = new StringBuilder();
                    ayuda.Append("Desde esta ventana puede crear una nueva hoja de gastos con sus diferentes detalles de gasto.");
                    ayuda.Append(" Para crear, eliminar, o ver detalles de gastos utilice las botones 'Nuevo detalle', 'Eliminar detalle', 'Ver detalle'.");
                    ayuda.Append(" Una vez creados todos los detalles necesarios puede guardar la hoja de gastos seleccionando la opción 'Guardar' o salir sin guardar los cambios seleccionando la opción 'Atrás'.");
                    ayuda.Append(" Una vez guardada una hoja de gastos ya no podrá ser modificada.");
                    MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
                    break;
                case Estados.NavegacionEmpleado:
                    var ayuda2 = new StringBuilder();
                    ayuda2.Append("Desde esta ventana puede ver una hoja de gastos creada anteriormente, al igual que revisar cada uno de los detalles de gasto.");
                    ayuda2.Append(" Para ver un detalle de gasto seleccione la opción 'Ver detalle'. Para salir seleccione la opción 'Atrás'.");
                    MessageBox.Show(ayuda2.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
                    break;
                case Estados.NavegacionResponsable:
                    var ayuda3 = new StringBuilder();
                    ayuda3.Append("Desde esta ventana puede ver una hoja de gastos creada por cualquier empleado de su departamento, al igual que revisar cada uno de los detalles de gasto.");
                    ayuda3.Append(" Para ver un detalle de gasto seleccione la opción 'Ver detalle'. También puede aprobar o rechazar la hoja seleccionando la opción 'Aprobar/Rechazar'. Para salir seleccione la opción 'Atrás'.");
                    MessageBox.Show(ayuda3.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
                    break;
                case Estados.NavegacionResponsableHojaGestionada:
                    var ayuda4 = new StringBuilder();
                    ayuda4.Append("Desde esta ventana puede ver una hoja de gastos creada por cualquier empleado de su departamento, al igual que revisar cada uno de los detalles de gasto.");
                    ayuda4.Append(" Para ver un detalle de gasto seleccione la opción 'Ver detalle'. Para salir seleccione la opción 'Atrás'.");
                    MessageBox.Show(ayuda4.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
                    break;
                default:
                    break;
            }
            
        }

        private void btnEliminarDetalle_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.EliminarDetalle);
        }

        private void btnNuevoDetalle_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.NuevoDetalle);
        }

        private void btnVerDetalle_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.MostrarDetalle);
        }

        private void btnGuardar_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.GuardarHoja);
        }

        private void btnAceptarRechazar_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.CambiarEstadoHoja);
        }

        private void btnAtras_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Atras);
        }

        private void btnAyuda_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Ayuda);
        }


    }
}
