﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GGP.Gestion
{
    public partial class frmListaHojasGastos : Form
    {
        private Estados _estado;
        private Negocio.FachadaNegocio.SesionUsuario _sesion;
        
        public frmListaHojasGastos()
        {
            InitializeComponent();

            _sesion = Negocio.FachadaNegocio.SesionUsuario.Instanciar();

            switch (_sesion.Rol)
            {
                case Negocio.Modulos.TipoRolUsuario.Administrador:
                    MessageBox.Show("No tiene permisos para acceder a este mantenimiento.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    this.Close();
                    break;
                case Negocio.Modulos.TipoRolUsuario.Empleado:
                    _estado = Estados.NavegacionEmpleado;
                    break;
                case Negocio.Modulos.TipoRolUsuario.Responsable:
                    _estado = Estados.NavegacionResponsable;
                    break;
                default:
                    break;
            }

            RealizarAccion(Acciones.CargarFormulario);
        }

        private enum Estados
        {
            NavegacionEmpleado,
            NavegacionResponsable
        }

        private enum Acciones
        {
            CargarFormulario,
            FiltrarHojas,
            MostrarHoja,
            Atras,
            Ayuda
        }

        private void RealizarAccion(Acciones accion)
        {
            try
            {
                switch (_estado)
                {
                    case Estados.NavegacionEmpleado:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario(false);
                                break;
                            case Acciones.MostrarHoja:
                                RealizarMostrarHoja();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyuda();
                                break;
                            default:
                                break;
                        }
                        break;

                    case Estados.NavegacionResponsable:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario(true);
                                break;
                            case Acciones.FiltrarHojas:
                                RealizarFiltrarHojas();
                                break;
                            case Acciones.MostrarHoja:
                                RealizarMostrarHoja();
                                break;
                            case Acciones.Atras:
                                RealizarAtras();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyuda();
                                break;
                            default:
                                break;
                        }
                        break;

                    default:
                        break;
                }

                HabilitarControles();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void HabilitarControles()
        {
            switch (_estado)
            {
                case Estados.NavegacionEmpleado:
                    lblFiltro.Visible = false;
                    cmbFiltro.Visible = false;
                    btnFiltrar.Visible = false;
                    break;

                case Estados.NavegacionResponsable:
                    lblFiltro.Visible = true;
                    cmbFiltro.Visible = true;
                    btnFiltrar.Visible = true;
                    break;

                default:
                    break;
            }
        }

        private void RealizarCargarFormulario(bool mostrarTodosLosEmpleados)
        {
            ICollection<Negocio.Modulos.HojaGastos> hojasGastos;

            if (mostrarTodosLosEmpleados)
                hojasGastos = Negocio.FachadaNegocio.GestionHojasGastos.ObtenerHojasGastosDepartamento(_sesion.IdDepartamento);
            else
                hojasGastos = Negocio.FachadaNegocio.GestionHojasGastos.ObtenerHojasGastos(_sesion.IdUsuario);

            grdHojasGastos.DataSource = hojasGastos;

            var sesion = Negocio.FachadaNegocio.SesionUsuario.Instanciar();
            var empleadosDepartamento = Negocio.FachadaNegocio.GestionUsuarios.ObtenerUsuarios(sesion.IdDepartamento);
            empleadosDepartamento.Add(new Negocio.Modulos.Usuario(null,"","","","",true,0,Negocio.Modulos.TipoRolUsuario.Empleado));
            usuarioBindingSource.DataSource = empleadosDepartamento.OrderBy(x => x.NombreApellidos);
        }

        private void RealizarFiltrarHojas()
        {
            ICollection<Negocio.Modulos.HojaGastos> hojasGastos;
            if (cmbFiltro.SelectedValue == null)
                hojasGastos = Negocio.FachadaNegocio.GestionHojasGastos.ObtenerHojasGastosDepartamento(_sesion.IdDepartamento);
            else
                hojasGastos = Negocio.FachadaNegocio.GestionHojasGastos.ObtenerHojasGastos((int)cmbFiltro.SelectedValue);
                
            grdHojasGastos.DataSource = hojasGastos;
        }

        private void RealizarMostrarHoja()
        {
            if (grdHojasGastos.SelectedRows.Count == 1)
            {
                var frmDetalleHoja = new DetalleHojaGastos((int)grdHojasGastos.SelectedRows[0].Cells["idHojaGastosDataGridViewTextBoxColumn"].Value);
                frmDetalleHoja.ShowDialog();

                RealizarAccion(Acciones.CargarFormulario);
            }
            else
                MessageBox.Show("Debe seleccionar una hoja de gastos.", "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            
        }

        private void RealizarAtras()
        {
            this.Close();
        }

        private void RealizarAyuda()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Desde esta ventana puede ver un listado de hojas de gastos. En caso de ser responsable de departamento podrá seleccionar un empleado y posteriormente seleccionar la opción 'Filtrar' para ver las hojas de gasto de un solo empleado.");
            ayuda.Append(" Para ver el detalle de una hoja de gastos seleccione la hoja en la lista y después seleccione la opción 'Detalle'. Para salir seleccione la opción 'Atrás'.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void btnFiltrar_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.FiltrarHojas);
        }

        private void btnDetalle_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.MostrarHoja);
        }

        private void btnAtras_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Atras);
        }

        private void btnAyuda_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Ayuda);
        }



    }
}
