﻿using GGP.Administracion.Calendarios;
using GGP.Administracion.Departamentos;
using GGP.Administracion.EmpleadosRoles;
using GGP.Administracion.TiposGastos;
using GGP.Gestion;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GGP
{
    public partial class frmMenu : Form
    {
        private Estados _estado;

        public frmMenu()
        {
            InitializeComponent();

        }

        private enum Estados
        {
            NoLogin,
            NavegacionAdministrador,
            NavegacionEmpleado,
            NavegacionResponsable
        }

        private enum Acciones
        {
            CargarFormulario,
            GestionDepartamentos,
            GestionUsuariosRoles,
            GestionTiposGasto,
            GestiónCalendarios,
            NuevaHojaGastos,
            VerHojasGastos,
            Salir,
            Ayuda
        }

        private void RealizarAccion(Acciones accion)
        {
            try
            {
                switch (_estado)
                {
                    case Estados.NoLogin:
                        switch (accion)
                        {
                            case Acciones.CargarFormulario:
                                RealizarCargarFormulario();
                                break;

                            case Acciones.Salir:
                                RealizarSalir();
                                break;

                            default:
                                break;
                        }
                        break;

                    case Estados.NavegacionAdministrador:
                        switch (accion)
                        {
                            case Acciones.GestionDepartamentos:
                                RealizarGestionDepartamentos();
                                break;
                            case Acciones.GestionUsuariosRoles:
                                RealizarGestionUsuariosRoles();
                                break;
                            case Acciones.GestionTiposGasto:
                                RealizarGestionTiposGasto();
                                break;
                            case Acciones.GestiónCalendarios:
                                RealizarGestiónCalendarios();
                                break;
                            case Acciones.Salir:
                                RealizarSalir();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyuda();
                                break;
                            default:
                                break;
                        }
                        break;

                    case Estados.NavegacionEmpleado:
                        switch (accion)
                        {
                            case Acciones.NuevaHojaGastos:
                                RealizarNuevaHojaGastos();
                                break;
                            case Acciones.VerHojasGastos:
                                RealizarVerHojasGastos();
                                break;
                            case Acciones.Salir:
                                RealizarSalir();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyuda();
                                break;
                            default:
                                break;
                        }
                        break;

                    case Estados.NavegacionResponsable:
                        switch (accion)
                        {
                            case Acciones.NuevaHojaGastos:
                                RealizarNuevaHojaGastos();
                                break;
                            case Acciones.VerHojasGastos:
                                RealizarVerHojasGastos();
                                break;
                            case Acciones.Salir:
                                RealizarSalir();
                                break;
                            case Acciones.Ayuda:
                                RealizarAyuda();
                                break;
                            default:
                                break;
                        }
                        break;

                    default:
                        break;
                }

                HabilitarControles();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Atención", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void HabilitarControles()
        {
            switch (_estado)
            {
                case Estados.NavegacionAdministrador:
                    pnlAdministrador.Visible = true;
                    btnGestionDepartamentos.Visible = true;
                    btnGestionUsuarios.Visible = true;
                    btnTiposGastos.Visible = true;
                    btnCalendarios.Visible = true;
                    pnlUsuario.Visible = false;
                    pnlAdministrador.Location = new Point(25, 25);
                    btnSalir.Visible = true;
                    btnAyuda.Visible = true;
                    break;

                case Estados.NavegacionEmpleado:
                    pnlAdministrador.Visible = false;
                    pnlUsuario.Visible = true;
                    btnNuevaHoja.Visible = true;
                    btnHojas.Visible = true;
                    pnlUsuario.Location = new Point(25, 25);
                    btnSalir.Visible = true;
                    btnAyuda.Visible = true;
                    break;

                case Estados.NavegacionResponsable:
                    pnlAdministrador.Visible = false;
                    pnlUsuario.Visible = true;
                    btnNuevaHoja.Visible = true;
                    btnHojas.Visible = true;
                    pnlUsuario.Location = new Point(25, 25);
                    btnSalir.Visible = true;
                    btnAyuda.Visible = true;
                    break;

                default:
                    break;
            }

        }

        private void RealizarCargarFormulario()
        {
            var session = Negocio.FachadaNegocio.SesionUsuario.Instanciar();
            if (session.IdUsuario == 0)
            {
                var frmLogin = new Login();
                frmLogin.ShowDialog();

                EstablerEstado();

                if (_estado == Estados.NoLogin)
                    RealizarAccion(Acciones.Salir);
            }
        }



        private void RealizarGestionDepartamentos()
        {
            var frmGestionDepartamentos = new frmGestionDepartamentos();
            frmGestionDepartamentos.ShowDialog();
        }

        private void RealizarGestionUsuariosRoles()
        {
            var frmGestionEmpleadosRoles = new GestionEmpleadosRoles();
            frmGestionEmpleadosRoles.ShowDialog();
        }

        private void RealizarGestionTiposGasto()
        {
            var frmGestionTiposGastos = new GestionTiposGastos();
            frmGestionTiposGastos.ShowDialog();
        }

        private void RealizarGestiónCalendarios()
        {
            var frmGestionCalendarios = new frmGestionCalendarios();
            frmGestionCalendarios.ShowDialog();
        }

        private void RealizarNuevaHojaGastos()
        {
            var frmDetalleHojaGastos = new DetalleHojaGastos(null);
            frmDetalleHojaGastos.ShowDialog();
        }

        private void RealizarVerHojasGastos()
        {
            var frmListaHojasGastos = new frmListaHojasGastos();
            frmListaHojasGastos.ShowDialog();
        }

        private void RealizarSalir()
        {
            if (_estado != Estados.NoLogin)
            {
                var seleccionSalir = MessageBox.Show("Desea salir de la aplicación?", "Atención", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (seleccionSalir == System.Windows.Forms.DialogResult.Yes)
                    this.Close();
            }
            else
                this.Close();
        }

        private void RealizarAyuda()
        {
            var ayuda = new StringBuilder();
            ayuda.Append("Seleccione la opción que desea realizar.");
            ayuda.Append(" Las opciones disponibles variarán dependiendo de su perfil de usuario.");
            MessageBox.Show(ayuda.ToString(), "Ayuda", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        private void EstablerEstado()
        {
            var session = Negocio.FachadaNegocio.SesionUsuario.Instanciar();

            switch (session.Rol)
            {
                case Negocio.Modulos.TipoRolUsuario.Administrador:
                    _estado = Estados.NavegacionAdministrador;
                    break;
                case Negocio.Modulos.TipoRolUsuario.Empleado:
                    _estado = Estados.NavegacionEmpleado;
                    break;
                case Negocio.Modulos.TipoRolUsuario.Responsable:
                    _estado = Estados.NavegacionResponsable;
                    break;
                default:
                    _estado = Estados.NoLogin;
                    break;
            }
        }

        private void btnGestionDepartamentos_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.GestionDepartamentos);
        }

        private void btnGestionUsuarios_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.GestionUsuariosRoles);
        }

        private void btnTiposGastos_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.GestionTiposGasto);
        }

        private void btnCalendarios_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.GestiónCalendarios);
        }

        private void btnNuevaHoja_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.NuevaHojaGastos);
        }

        private void btnHojas_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.VerHojasGastos);
        }

        private void btnSalir_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Salir);
        }

        private void btnAyuda_Click(object sender, EventArgs e)
        {
            RealizarAccion(Acciones.Ayuda);
        }

        private void frmMenu_Shown(object sender, System.EventArgs e)
        {
            var session = Negocio.FachadaNegocio.SesionUsuario.Instanciar();

            if (session.IdUsuario != 0)
            {
                EstablerEstado();
            }
            else
                _estado = Estados.NoLogin;


            RealizarAccion(Acciones.CargarFormulario);
        }

    }
}
