﻿using Negocio.Modulos;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Negocio.FachadaNegocio
{
    public class GestionDetallesHojaGastos
    {
        private static Datos.AccesoDatos.DetallesHojaGastos _miAccesoDatos;
        private static Datos.AccesoDatos.DetallesHojaGastos MiAccesoDatos
        {
            get
            {
                if (_miAccesoDatos == null)
                    _miAccesoDatos = Datos.AccesoDatos.DetallesHojaGastos.Instanciar();

                return _miAccesoDatos;
            }
        }

        public static ICollection<DetalleHojaGastos> ObtenerDetallesHojaGastos(int idHojaGastos)
        {
            var detallesHojaGastosPOCO = MiAccesoDatos.ObtenerDetallesHojaGastos(idHojaGastos);
            var detallesHojaGastos = new Collection<DetalleHojaGastos>();

            foreach (var detalleHojaGastosPOCO in detallesHojaGastosPOCO)
            {
                detallesHojaGastos.Add(new DetalleHojaGastos(detalleHojaGastosPOCO.idDetalleHojaGastos, detalleHojaGastosPOCO.idHojaGastos, detalleHojaGastosPOCO.idTipoGasto, detalleHojaGastosPOCO.descripcion, detalleHojaGastosPOCO.observaciones,
                    detalleHojaGastosPOCO.fechaInicio, detalleHojaGastosPOCO.fechaFin, detalleHojaGastosPOCO.importe, detalleHojaGastosPOCO.importeCalculado, detalleHojaGastosPOCO.imagenAdjunta));
            }

            return detallesHojaGastos;
        }

        public static DetalleHojaGastos ObtenerDetalleHojaGastos(int idDetalleHojaGastos)
        {
            var detalleHojaGastosPOCO = MiAccesoDatos.ObtenerDetalleHojaGastos(idDetalleHojaGastos);

            if (detalleHojaGastosPOCO != null)
            {
                return new DetalleHojaGastos(detalleHojaGastosPOCO.idDetalleHojaGastos, detalleHojaGastosPOCO.idHojaGastos, detalleHojaGastosPOCO.idTipoGasto, detalleHojaGastosPOCO.descripcion, detalleHojaGastosPOCO.observaciones,
                    detalleHojaGastosPOCO.fechaInicio, detalleHojaGastosPOCO.fechaFin, detalleHojaGastosPOCO.importe, detalleHojaGastosPOCO.importeCalculado, detalleHojaGastosPOCO.imagenAdjunta);
            }
            else
            {
                return null;
            }
            
        }

        public static void CrearDetalleHojaGastos(DetalleHojaGastos detalleHojaGastos)
        {
            if (detalleHojaGastos.FechaFin < detalleHojaGastos.FechaInicio)
            {
                throw new Exception("La fecha de fin debe ser igual o superior a la fecha de inicio.");
            }
            MiAccesoDatos.CrearDetalleHojaGastos(detalleHojaGastos.IdHojaGastos, detalleHojaGastos.IdTipoGasto, detalleHojaGastos.Descripcion,
                detalleHojaGastos.Observaciones, detalleHojaGastos.FechaInicio, detalleHojaGastos.FechaFin, detalleHojaGastos.Importe, detalleHojaGastos.ImporteCalculado, detalleHojaGastos.ImagenAdjunta);
        }

        public static double CalcularImporte(int idDepartamento, double importe, DateTime fechaInicio, DateTime fechaFin)
        {
            double importeTotal = 0;
            double importePorDia = importe / ((fechaFin - fechaInicio).TotalDays+1);

            var departamento = GestionDepartamentos.ObtenerDepartamento(idDepartamento);
            double factorFestivo = 1 + ((double)departamento.PorcentajePlusFestivo / 100);

            var fecha = fechaInicio;
            while (fecha <= fechaFin)
            {
                if (Negocio.FachadaNegocio.GestionFestivos.ObtenerFestivo(fecha) != null)
                    importeTotal += importePorDia * factorFestivo;
                else
                    importeTotal += importePorDia;
                fecha = fecha.AddDays(1);
            }

            return importeTotal;
        }
    }
}
