﻿using Negocio.Modulos;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Negocio.FachadaNegocio
{
    public class GestionHojasGastos
    {
        private static Datos.AccesoDatos.HojasGastos _miAccesoDatos;
        private static Datos.AccesoDatos.HojasGastos MiAccesoDatos
        {
            get
            {
                if (_miAccesoDatos == null)
                    _miAccesoDatos = Datos.AccesoDatos.HojasGastos.Instanciar();

                return _miAccesoDatos;
            }
        }

        public static ICollection<HojaGastos> ObtenerHojasGastos(int idUsuario)
        {
            var hojasGastosPOCO = MiAccesoDatos.ObtenerHojasGastos(idUsuario);
            var hojasGastos = new Collection<HojaGastos>();

            foreach (var hojaGastosPOCO in hojasGastosPOCO)
            {
                hojasGastos.Add(new HojaGastos(hojaGastosPOCO.idHojaGastos, hojaGastosPOCO.idUsuario, TraducirEstado(hojaGastosPOCO.estado) , hojaGastosPOCO.idResponsable, 
                    hojaGastosPOCO.fechaCreacion,hojaGastosPOCO.comentarioResponsable, hojaGastosPOCO.fechaCambioEstado));
            }

            return hojasGastos;
        }

        public static ICollection<HojaGastos> ObtenerHojasGastosDepartamento(int idDepartamento)
        {
            var usuarios = FachadaNegocio.GestionUsuarios.ObtenerUsuarios(idDepartamento);
            var hojasGastos = new Collection<HojaGastos>();

            foreach (var usuario in usuarios)
            {
                foreach (var hojaGastos in ObtenerHojasGastos(usuario.IdUsuario.GetValueOrDefault()))
                {
                    hojasGastos.Add(hojaGastos);
                } 
            }

            return hojasGastos;
        }

        public static HojaGastos ObtenerHojaGastos(int idHojaGastos)
        {
            var hojaGastosPOCO = MiAccesoDatos.ObtenerHojaGastos(idHojaGastos);

            if (hojaGastosPOCO != null)
                return new HojaGastos(hojaGastosPOCO.idHojaGastos, hojaGastosPOCO.idUsuario, TraducirEstado(hojaGastosPOCO.estado), hojaGastosPOCO.idResponsable,
                    hojaGastosPOCO.fechaCreacion, hojaGastosPOCO.comentarioResponsable, hojaGastosPOCO.fechaCambioEstado);
            else
                return null;
        }

        private static EstadoHojaGastos TraducirEstado(int estado)
        {
            switch (estado)
            {
                case 1:
                    return EstadoHojaGastos.Pendiente;
                case 2:
                    return EstadoHojaGastos.Aceptada;
                case 3:
                    return EstadoHojaGastos.Rechazada;
                default:
                    throw new Exception("Se ha producido un error en la aplicáción. Informe a su administrador del sistema con el código de error E001.");
            }
        }

        public static void CrearHojaGastos(HojaGastos hojaGastos)
        {
            MiAccesoDatos.CrearHojaGastos(hojaGastos.IdUsuario, (int)hojaGastos.Estado, hojaGastos.IdResponsable, 
                hojaGastos.FechaCreacion, hojaGastos.ComentarioResponsable, hojaGastos.FechaCambioEstado);
        }

        public static void ModificarHojaGastos(HojaGastos hojaGastos)
        {
            MiAccesoDatos.ModificarHojaGastos(hojaGastos.IdHojaGastos.GetValueOrDefault(), (int)hojaGastos.Estado, hojaGastos.IdResponsable,
                hojaGastos.ComentarioResponsable, hojaGastos.FechaCambioEstado);
        }

        public static HojaGastos ObtenerUltimaHojaGastos(int idUsuario)
        {
            var hojaGastosPOCO = MiAccesoDatos.ObtenerUltimaHojaGastos(idUsuario);
            if (hojaGastosPOCO != null)
                return new HojaGastos(hojaGastosPOCO.idHojaGastos, hojaGastosPOCO.idUsuario, TraducirEstado(hojaGastosPOCO.estado), hojaGastosPOCO.idResponsable,
                    hojaGastosPOCO.fechaCreacion, hojaGastosPOCO.comentarioResponsable, hojaGastosPOCO.fechaCambioEstado);
            else
                return null;
        }
    }
}
