﻿using Negocio.Modulos;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Negocio.FachadaNegocio
{
    public class GestionUsuarios
    {
        private static Datos.AccesoDatos.Usuarios _miAccesoDatos;
        public static Datos.AccesoDatos.Usuarios MiAccesoDatos
        {
            get 
            {
                if (_miAccesoDatos == null)
                    _miAccesoDatos = Datos.AccesoDatos.Usuarios.Instanciar();

                return _miAccesoDatos;
            }
        }

        public static ICollection<Usuario> ObtenerUsuarios()
        {
            var usuariosPOCO = MiAccesoDatos.ObtenerUsuarios();
            var usuarios = new Collection<Usuario>();

            foreach (var usuarioPOCO in usuariosPOCO)
            {
                usuarios.Add(new Usuario(usuarioPOCO.idUsuario, usuarioPOCO.nombre, usuarioPOCO.apellidos, usuarioPOCO.nombreLogin, usuarioPOCO.contrasena,
                    usuarioPOCO.activo, usuarioPOCO.idDepartamento, TraducirRol(usuarioPOCO.rol)));
            }

            return usuarios;
        }

        public static ICollection<Usuario> ObtenerUsuarios(int idDepartamento)
        {
            var usuariosPOCO = MiAccesoDatos.ObtenerUsuarios(idDepartamento);
            var usuarios = new Collection<Usuario>();

            foreach (var usuarioPOCO in usuariosPOCO)
            {
                usuarios.Add(new Usuario(usuarioPOCO.idUsuario, usuarioPOCO.nombre, usuarioPOCO.apellidos, usuarioPOCO.nombreLogin, usuarioPOCO.contrasena,
                    usuarioPOCO.activo, usuarioPOCO.idDepartamento, TraducirRol(usuarioPOCO.rol)));
            }

            return usuarios;
        }

        public static Usuario ObtenerUsuario(int idUsuario)
        {
            var usuarioPOCO = MiAccesoDatos.ObtenerUsuario(idUsuario);

            if (usuarioPOCO == null)
                return null;
            else
                return new Usuario(usuarioPOCO.idUsuario, usuarioPOCO.nombre, usuarioPOCO.apellidos, usuarioPOCO.nombreLogin, usuarioPOCO.contrasena,
                        usuarioPOCO.activo, usuarioPOCO.idDepartamento, TraducirRol(usuarioPOCO.rol));
        }

        public static Usuario ObtenerUsuario(String nombreLogin)
        {
            var usuarioPOCO = MiAccesoDatos.ObtenerUsuario(nombreLogin);

            return new Usuario(usuarioPOCO.idUsuario, usuarioPOCO.nombre, usuarioPOCO.apellidos, usuarioPOCO.nombreLogin, usuarioPOCO.contrasena,
                    usuarioPOCO.activo, usuarioPOCO.idDepartamento, TraducirRol(usuarioPOCO.rol));
        }

        private static TipoRolUsuario TraducirRol(int rol)
        {
            switch (rol)
            {
                case 1:
                    return TipoRolUsuario.Administrador;
                case 2:
                    return TipoRolUsuario.Empleado;
                case 3:
                    return TipoRolUsuario.Responsable;
                default:
                    throw new Exception("Se ha producido un error en la aplicáción. Informe a su administrador del sistema con el código de error E002.");
            }
        }

        public static void CrearUsuario(Usuario usuario)
        {
           MiAccesoDatos.CrearUsuario(usuario.Nombre, usuario.Apellidos, usuario.NombreLogin, usuario.Contrasena, usuario.Activo, usuario.IdDepartamento, (int)usuario.Rol);
        }

        public static void ModificarUsuario(Usuario usuario)
        {
            MiAccesoDatos.ModificarUsuario(usuario.IdUsuario.GetValueOrDefault(), usuario.Nombre, usuario.Apellidos, 
                usuario.NombreLogin, usuario.Contrasena, usuario.Activo, usuario.IdDepartamento, (int)usuario.Rol);
        }

        public static bool ComprobarLogin(string nombreLogin, string contrasena)
        {
            var usuarioPOCO = MiAccesoDatos.ObtenerUsuario(nombreLogin);

            if (usuarioPOCO != null && usuarioPOCO.nombreLogin == nombreLogin && usuarioPOCO.contrasena == contrasena && usuarioPOCO.activo == true)
                return true;
            else
                return false;
        }

        public static bool LoginRepetido(int idUsuario, string nombreLogin)
        {
            var usuarioPOCO = MiAccesoDatos.ObtenerUsuario(nombreLogin);

            if (usuarioPOCO == null)
                return false;
            else if (usuarioPOCO.idUsuario == idUsuario)
                return false;
            else
                return true;
        }
    }
}
